'use strict';

/* Requirements  ----------------------------------------------------------------- */
var gulp = require('gulp');
var sass = require('gulp-sass');
var sassGlob = require('gulp-sass-glob');
var sourcemaps = require('gulp-sourcemaps');
var autoprefixer = require('gulp-autoprefixer');
var sassdoc = require('sassdoc');
var jshint = require('gulp-jshint');
var concat = require('gulp-concat');
var uglify = require('gulp-uglify');
var browserify = require('gulp-browserify');
var copy = require('copy');
var _ = require('underscore');
var urlAdjuster = require('gulp-css-url-adjuster');

/* Settings  ----------------------------------------------------------------- */
var sassfiles = './resources/assets/sass/**/*.scss';
var cssoutput = './dist/';
var jsfiles = 'resources/assets/js/**/*.js';
var jsoutput = './dist/';

/* Default  ----------------------------------------------------------------- */
gulp.task('default', ['watch']);

/* Build  -----------------------------------------------------------------
 Before releasing or deploying always run build task to make sure all assets
 have been copied and all css has been compiled and included.
*/
gulp.task('build', [
    'sync-resources',
    'sync-css-resources',
    'sass',
    'jshint',
    'build-js'
]);

/* Watch  -----------------------------------------------------------------
 When running watch gulp will automatically watch these folders for changes
 and run the proper tasks when files change.
*/
gulp.task('watch', function() {
    gulp.watch(sassfiles, ['sass'])
        .on('change', function(event) {
            console.log('File ' + event.path + ' was ' + event.type + ', running tasks...');
        });

    gulp.watch(jsfiles, ['jshint','build-js']);
});

/* SYNC Resources -------------------------------------------------------
 Define all your modules here that you have included with npm. Assets like images will be automatically moved
 to a nice folder and urls will be adjusted automatically for your.
 */
var resource_image_files = {
    sliderpro: ['./node_modules/slider-pro/dist/css/images/**', './dist/images/sliderpro'],
    photoswipe: ['./node_modules/photoswipe/dist/default-skin/*.{png,jpg,svg,gif}', './dist/images/photoswipe']
};

var resource_css_files = {
    sliderpro: [
        {source: './node_modules/slider-pro/dist/css/slider-pro.css', replace_path: 'images/'}
    ],
    photoswipe: [
        {source: './node_modules/photoswipe/dist/default-skin/default-skin.css', replace_path: ''},
        {source: './node_modules/photoswipe/dist/photoswipe.css', replace_path: false}
    ]
};

gulp.task('sync-resources', function() {

    _.each(resource_image_files, function(value, key){
        var source = value[0];
        var dest = value[1];
        copy(source, dest,function(err, files){});
    });

});

gulp.task('sync-css-resources', function() {

    _.each(resource_css_files, function(files, module_name){
        _.each(files, function(value){

            var gulpAdjuster = gulp.src(value.source);

            if (value.replace_path !== false) {
                gulpAdjuster.pipe(urlAdjuster({
                    replace:  [value.replace_path,'./images/'+module_name+'/'],
                }));
            }

            gulpAdjuster.pipe(gulp.dest('resources/modules/'+module_name));
        });

    });

});

/* SASS -----------------------------------------------------------------
 Define all your sass files here. Important if you add new modules that have
 css or sass files add them to the includepath over here.
 */
var sassOptions = {
    errLogToConsole: true,
    outputStyle: 'compressed',
    includePaths: [
        './resources/modules/photoswipe',
        './resources/modules/sliderpro',
    ]
};

gulp.task('sass', function () {
    return gulp
        .src(sassfiles)
        .pipe(sourcemaps.init())
        .pipe(sassGlob())
        .pipe(sass(sassOptions).on('error', sass.logError))
        .pipe(sourcemaps.write())
        //.pipe(autoprefixer())
        .pipe(gulp.dest(cssoutput))
});

var sassdocOptions = {
    dest: './public/sassdoc'
};

gulp.task('sassdoc', function () {
    return gulp
        .src(sassfiles)
        .pipe(sassdoc(sassdocOptions))
        .resume();
});

/* Javascript -----------------------------------------------------------------
 This part creates the javascript files for you. Use require (supported because of
 the browserfy plugin) to load the module when needed.
 */
gulp.task('jshint', function() {
    return gulp.src(jsfiles)
        .pipe(jshint())
        .pipe(jshint.reporter('jshint-stylish'));
});

gulp.task('build-js', function() {
    return gulp.src(jsfiles)
        .pipe(sourcemaps.init())
        .pipe(concat('index.js'))
        //only uglify if gulp is ran with '--type production'
        .pipe(browserify())
        .pipe(uglify())
        .pipe(sourcemaps.write())
        .pipe(gulp.dest(jsoutput));
});